"""Ignition API Database - Function definitions for code intelligence."""

__all__ = ["API_DB_DIR"]

from pathlib import Path

# Directory containing API definition JSON files
API_DB_DIR = Path(__file__).parent
